import { defineComponent, ref } from 'vue'
import { Button, Checkbox, Divider, message, Modal } from 'ant-design-vue'
import useSearchConfigStore from './useSearchConfigStore'
import { EditOutlined, DeleteOutlined, PlusOutlined } from '@ant-design/icons-vue'
import asyncLoader from '@/utils/asyncLoader'
const SearchAdder = asyncLoader(() => import('./SearchAdder'))
const SearchItem = defineComponent({
  props: {
    icon: {
      type: String,
      default: ''
    },
    name: {
      type: String,
      default: ''
    },
    url: {
      type: String,
      default: ''
    },
    editable: {
      type: Boolean,
      default: false
    },
    modelValue: {
      type: Boolean,
      default: false
    }
  },
  emits: ['update:modelValue', 'edit'],
  setup(props, ctx) {
    const searchConfig = useSearchConfigStore()
    return () => (
      <div
        key={props.url}
        class={
          'flex justify-between items-center py-2 px-4 cursor-pointer transition-all ' +
          (searchConfig.current.url === props.url ? 'bg-blue-100 rounded' : '')
        }
        onClick={() => {
          searchConfig.current = {
            icon: props.icon,
            name: props.name,
            url: props.url,
            show: true
          }
        }}
      >
        <div class="flex items-center">
          <div
            class="w-6 h-6 mr-2 bg-center bg-contain bg-no-repeat"
            style={{
              backgroundImage: `url('${props.icon}')`
            }}
          />
          <div class="text-black/80">
            {props.name}{' '}
            {searchConfig.current.url === props.url && (
              <span class="text-black/60">（当前使用）</span>
            )}
          </div>
        </div>
        <div class="flex items-center" onClick={(e) => e.stopPropagation()}>
          <Checkbox
            checked={props.modelValue}
            onChange={(e) => {
              const checked = e.target.checked
              if (!checked && props.url === searchConfig.current.url) {
                message.warning('不可隐藏当前使用搜索引擎')
              } else {
                ctx.emit('update:modelValue', e.target.checked)
              }
            }}
          />
          {props.editable && (
            <>
              <Button
                size="small"
                type="link"
                class="ml-4"
                icon={<EditOutlined />}
                onClick={() => {
                  ctx.emit('edit', {
                    icon: props.icon,
                    name: props.name,
                    url: props.url
                  })
                }}
              ></Button>
              <Button
                size="small"
                type="text"
                danger
                class="ml-2"
                icon={<DeleteOutlined />}
                onClick={() => {
                  if (props.url === searchConfig.current.url) {
                    message.warning('不可删除当前使用搜索引擎')
                  } else {
                    const idx = searchConfig.customList.findIndex((el) => el.url === props.url)
                    searchConfig.customList.splice(idx, 1)
                  }
                }}
              ></Button>
            </>
          )}
        </div>
      </div>
    )
  }
})

export default defineComponent(() => {
  const searchConfig = useSearchConfigStore()
  const showAdder = ref<{ [key: string]: any } | null | undefined>(undefined)
  return () => (
    <div
      class="w-full h-full bg-white/80 backdrop-blur p-4 flex flex-col select-text"
      onContextmenu={(e) => e.stopPropagation()}
    >
      <h2 class="text-center tracking-widest font-bold text-xl text-black/80">管理搜索引擎</h2>
      <div class="grid grid-cols-2 gap-4">
        <Divider>默认</Divider>
        <Divider>自定义</Divider>
      </div>
      <div class="w-full h-0 flex-grow grid grid-cols-2 gap-4">
        <div class="w-full h-full overflow-auto">
          {searchConfig.defaultList.map((el) => (
            <SearchItem key={el.url} url={el.url} icon={el.icon} name={el.name} v-model={el.show} />
          ))}
        </div>
        <div class="w-full h-full flex flex-col">
          <div class="w-full h-0 flex-grow overflow-y-auto">
            {searchConfig.customList.map((el) => (
              <SearchItem
                key={el.url}
                url={el.url}
                icon={el.icon}
                name={el.name}
                editable
                v-model={el.show}
                onEdit={(obj) => {
                  showAdder.value = obj
                }}
              />
            ))}
          </div>
          <Button
            type="primary"
            class="mt-4"
            block
            icon={<PlusOutlined />}
            onClick={() => {
              showAdder.value = null
            }}
          >
            添加自定义搜索引擎
          </Button>
        </div>
      </div>
      <Modal
        open={showAdder.value !== undefined}
        onUpdate:open={(e) => {
          if (!e) {
            showAdder.value = undefined
          }
        }}
        title="添加搜索引擎"
        footer={false}
      >
        <SearchAdder selected={showAdder.value as any} />
      </Modal>
    </div>
  )
})
